VERSION 5.00
Begin {C62A69F0-16DC-11CE-9E98-00AA00574A4F} FindDups 
   Caption         =   "Select Duplicate Records"
   ClientHeight    =   3600
   ClientLeft      =   45
   ClientTop       =   330
   ClientWidth     =   3600
   OleObjectBlob   =   "FindDups.frx":0000
   ShowModal       =   0   'False
   StartUpPosition =   1  'CenterOwner
End
Attribute VB_Name = "FindDups"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
Public Enum TableType
   NoType = 0
   IStandaloneTable = 1
   IFeatureLayer = 2
End Enum
Private m_pTable As ITable
Private m_TabType As TableType

' frmFindDups - Duplicate Record Tagger
' Selects duplicate records in selected table

Private Sub btnGetFields_Click()
   
   Dim pDoc As IMxDocument
   Dim pSelItem As IUnknown
   Dim TabType As TableType
   Dim pTable As ITable
   
   ' Check for selected table or feature class
   
   Set m_pTable = Nothing
   m_TabType = NoType
   lbFields.Clear
   cbVisExtent.Value = False
   
   Set pDoc = ThisDocument
   Set pSelItem = pDoc.SelectedItem
   If pSelItem Is Nothing Then
      MsgBox "Nothing selected", vbExclamation
      Exit Sub
   ElseIf TypeOf pSelItem Is IStandaloneTable Then
      TabType = IStandaloneTable
   ElseIf TypeOf pSelItem Is IFeatureLayer Then
      TabType = IFeatureLayer
   Else
      MsgBox "No table selected", vbExclamation
      Exit Sub
   End If
   Set pTable = pSelItem
   
   ' Populate field list
   
   Dim pFields As ITableFields
   Dim i As Long
   Dim pInfo As IFieldInfo
   Dim pField As iField
   Dim fType As esriFieldType
   
   Set pFields = pTable
   For i = 0 To pFields.FieldCount - 1
      Set pInfo = pFields.FieldInfo(i)
      If pInfo.Visible Then
         
         ' Check for supported field type
         
         Set pField = pTable.Fields.Field(i)
         fType = pField.Type
         If Not (fType = esriFieldTypeBlob Or _
            fType = esriFieldTypeGeometry Or _
            fType = esriFieldTypeRaster) Then
            
            lbFields.AddItem pField.AliasName
         
         End If
      End If
   Next
   If lbFields.ListCount = 0 Then
      MsgBox "No valid fields", vbExclamation
      Exit Sub
   End If
   
   If TabType = IStandaloneTable Then
      cbVisExtent.Enabled = False
   Else
      cbVisExtent.Enabled = True
   End If

   Set m_pTable = pTable
   m_TabType = TabType

End Sub

Private Sub btnDoIt_Click()

   If m_pTable Is Nothing Then
      MsgBox "No table selected", vbExclamation
      Exit Sub
   End If
   
   Dim pMouse As IMouseCursor
   Set pMouse = New MouseCursor
   pMouse.SetCursor 2
   
   ' Get selected field(s)
      
   Dim FList() As String, FieldString As String, FName As String
   Dim pField As iField
   Dim i As Long, n As Long, iField As Long
   
   FieldString = ""
   n = 0
   For i = 0 To lbFields.ListCount - 1
      If lbFields.Selected(i) Then
         iField = m_pTable.Fields.FindFieldByAliasName(lbFields.List(i))
         FName = m_pTable.Fields.Field(iField).Name
         ReDim Preserve FList(n)
         FList(n) = FName
         If FieldString = "" Then
            FieldString = FName
         Else
            FieldString = FieldString & ", " & FName
         End If
         n = n + 1
      End If
   Next
   If n = 0 Then
      MsgBox "No field(s) selected", vbExclamation
      Exit Sub
   End If
   
   ' Check for selection set
   
   Dim bUseSel As Boolean
   Dim pTSel As ITableSelection
   Dim pFSel As IFeatureSelection
   Dim pSS As ISelectionSet
   
   bUseSel = False
   If m_TabType = IStandaloneTable Then
      Set pTSel = m_pTable
      Set pSS = pTSel.SelectionSet
   ElseIf m_TabType = IFeatureLayer Then
      Set pFSel = m_pTable
      Set pSS = pFSel.SelectionSet
   End If
   i = pSS.Count
   bUseSel = (i > 0)
   
   ' Check for definition query
   
   Dim bUseQF As Boolean
   Dim pTD As ITableDefinition
   Dim WhereClause As String
   Dim pQF As IQueryFilter
   
   pUseQF = False
   If (Not bUseSel) Then
      Set pTD = m_pTable
      WhereClause = pTD.DefinitionExpression
      If WhereClause <> "" Then
         Set pQF = New QueryFilter
         pQF.WhereClause = WhereClause
         bUseQF = True
      End If
   End If
   
   ' Check for restrict to visible extent
   
   Dim bUseSF As Boolean
   Dim pDoc As IMxDocument
   Dim pMap As IMap
   Dim pActiveView As IActiveView
   Dim pViewExtent As IEnvelope
   Dim pPC As IPointCollection
   Dim pPoly As IPolygon
   Dim pSF As ISpatialFilter
   
   bUseSF = False
   If cbVisExtent.Value Then
      Set pDoc = ThisDocument
      Set pMap = pDoc.FocusMap
      Set pActiveView = pMap
      Set pViewExtent = pActiveView.Extent
      Set pPC = New Polygon
      With pPC
         .AddPoint pViewExtent.LowerLeft
         .AddPoint pViewExtent.UpperLeft
         .AddPoint pViewExtent.UpperRight
         .AddPoint pViewExtent.LowerRight
      End With
      Set pPoly = pPC
      pPoly.Close
      Set pPoly.SpatialReference = pMap.SpatialReference
      Set pSF = New SpatialFilter
      With pSF
         Set .Geometry = pPoly
         .GeometryField = "SHAPE"
         .SpatialRel = esriSpatialRelContains
         If bUseQF Then
            .WhereClause = WhereClause
         End If
      End With
      bUseSF = True
   End If
   
   ' Sort table
   
   Application.StatusBar.Message(0) = "Sorting..."
   DoEvents
   Dim pTS As ITableSort
   Set pTS = New TableSort
   With pTS
      .Fields = FieldString
      Set .Table = m_pTable
      For i = 0 To n - 1
         .Ascending(FList(i)) = True
         If cbCaseSensitive.Value Then
            .CaseSensitive(FList(i)) = True
         End If
      Next
      If bUseSF Then
         Set .QueryFilter = pSF
      ElseIf bUseQF Then
         Set .QueryFilter = pQF
      End If
      If bUseSel Then
         Set .SelectionSet = pSS
      End If
   End With
   pTS.Sort Nothing
   
   ' Loop through rows and find dups
   
   Dim pCursor As ICursor
   Dim pRow1 As iRow, pRow2 As iRow
   Dim pFeature As iFeature
   Dim bSelectAll As Boolean, bDup As Boolean, bFirst As Boolean
   Dim iNumRecs As Long, iNumDups As Long
   
   Application.StatusBar.Message(0) = "Analyzing..."
   DoEvents
   bSelectAll = cbSelectAll.Value
   Set pCursor = pTS.Rows
   Set pRow1 = pCursor.NextRow
   Set pRow2 = pCursor.NextRow
   iNumRecs = 0
   iNumDups = 0
   If m_TabType = IStandaloneTable Then
      pTSel.Clear
   ElseIf m_TabType = IFeatureLayer Then
      pFSel.Clear
   End If
   bFirst = True
   Do While Not pRow2 Is Nothing
   
      iNumRecs = iNumRecs + 1
      If iNumRecs Mod 100 = 0 Then
         Application.StatusBar.Message(0) = "Analyzing... " & iNumRecs
         DoEvents
      End If
      
      bDup = True
      For i = 0 To n - 1
         iField = m_pTable.FindField(FList(i))
         If pRow1.Value(iField) <> pRow2.Value(iField) Then
            bDup = False
            Exit For
         End If
      Next
      If bDup Then
         If m_TabType = IStandaloneTable Then
            If bFirst And bSelectAll Then
               pTSel.AddRow pRow1
            End If
            pTSel.AddRow pRow2
         ElseIf m_TabType = IFeatureLayer Then
            If bFirst And bSelectAll Then
               Set pFeature = pRow1
               pFSel.Add pFeature
            End If
            Set pFeature = pRow2
            pFSel.Add pFeature
         End If
         iNumDups = iNumDups + 1
         bFirst = False
      Else
         bFirst = True
      End If
      Set pRow1 = pRow2
      Set pRow2 = pCursor.NextRow
   
   Loop
   
   ' Wrap up
   
   If m_TabType = IStandaloneTable Then
      pTSel.SelectionChanged
      Set pSS = pTSel.SelectionSet
   ElseIf m_TabType = IFeatureLayer Then
      pFSel.SelectionChanged
      Set pSS = pFSel.SelectionSet
   End If
   i = pSS.Count
   
   MsgBox "Duplicates found: " & i, vbOK
   
End Sub
